Attribute VB_Name = "mdlPrint"
Option Explicit

Private Type Rect
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Type CharRange
    firstChar As Long       ' First character of range (0 for start of doc)
    lastChar As Long        ' Last character of range (-1 for end of doc)
End Type

Private Type FormatRange
    hdc As Long             ' Actual DC to draw on
    hdcTarget As Long       ' Target DC for determining text formatting
    rectRegion As Rect      ' Region of the DC to draw to (in twips)
    rectPage As Rect        ' Region of the entire DC (page size) (in twips)
    mCharRange As CharRange ' Range of text to draw (see above declaration)
End Type

Private Const WM_USER As Long = &H400
Private Const EM_FORMATRANGE As Long = WM_USER + 57
Private Const EM_SETTARGETDEVICE As Long = WM_USER + 72
Private Const PHYSICALOFFSETX As Long = 112
Private Const PHYSICALOFFSETY As Long = 113

Private Declare Function GetDeviceCaps Lib "gdi32" ( _
    ByVal hdc As Long, ByVal nIndex As Long) As Long
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
    (ByVal hwnd As Long, ByVal msg As Long, ByVal wp As Long, _
    LP As Any) As Long

Private Declare Function CreateDC Lib "gdi32" Alias "CreateDCA" _
    (ByVal lpDriverName As String, ByVal lpDeviceName As String, _
    ByVal lpOutput As Long, ByVal lpInitData As Long) As Long

Public Sub PrintRTF(RTF As RichTextBox, mFromPage As Integer, mToPage As Integer)
 On Error Resume Next
 Dim LeftOffset As Long, TopOffset As Long
 Dim LeftMargin As Long, TopMargin As Long
 Dim RightMargin As Long, BottomMargin As Long
 Dim fr As FormatRange
 Dim rcDrawTo As Rect
 Dim rectPage As Rect
 Dim TextLength As Long
 Dim NextCharPosition As Long
 Dim R As Long
 Dim mPage As Integer
 
 ' Start a print job to get a valid Printer.hDC
 ' Printer.Print Space(1)
 Printer.Orientation = PrnOrient

 ' Get the offsett to the printable area on the page in twips
 LeftOffset = Printer.ScaleX(GetDeviceCaps(Printer.hdc, PHYSICALOFFSETX), vbPixels, vbTwips)
 TopOffset = Printer.ScaleY(GetDeviceCaps(Printer.hdc, PHYSICALOFFSETY), vbPixels, vbTwips)
 ' Printer.KillDoc
 
 ' Calculate the Left, Top, Right, and Bottom margins
 LeftMargin = LeftM * 570 - LeftOffset
 TopMargin = TopM * 570 - TopOffset
 RightMargin = (Printer.Width - RightM * 570) - LeftOffset

 BottomMargin = (Printer.Height - BottomM * 570) - TopOffset

 ' Set printable area rect
 rectPage.Left = 0
 rectPage.Top = 0
 rectPage.Right = Printer.ScaleWidth
 rectPage.Bottom = Printer.ScaleHeight

 ' Set rect in which to print (relative to printable area)
 rcDrawTo.Left = LeftMargin
 rcDrawTo.Top = TopMargin
 rcDrawTo.Right = RightMargin
 rcDrawTo.Bottom = BottomMargin
 
 ' Set up the print instructions
 fr.hdc = Printer.hdc        ' Use the same DC for measuring and rendering
 fr.hdcTarget = Printer.hdc  ' Point at printer hDC
 fr.rectRegion = rcDrawTo    ' Indicate the area on page to draw to
 fr.rectPage = rectPage      ' Indicate entire size of page
 fr.mCharRange.firstChar = 0 ' Indicate start of text through
 fr.mCharRange.lastChar = -1 ' end of the text
 
 ' Get length of text in RTF
 TextLength = Len(RTF.Text)

 ' Loop printing each page until done
 mPage = 1
 Do
    If mPage = mFromPage Then
        Exit Do
    End If
    ' Print the page by sending EM_FORMATRANGE message
    NextCharPosition = SendMessage(RTF.hwnd, EM_FORMATRANGE, True, fr)
    If NextCharPosition >= TextLength Then Exit Do 'If done then exit
    fr.mCharRange.firstChar = NextCharPosition ' Starting position for next page
    fr.hdc = Printer.hdc
    fr.hdcTarget = Printer.hdc
    mPage = mPage + 1
 Loop

 ' Must cleanse memory here before print, otherwise font will not be right
 R = SendMessage(frmEdit.Text.hwnd, EM_FORMATRANGE, False, ByVal CLng(0))
    
 If NextCharPosition >= TextLength Then
    Exit Sub
 End If
    
 ' Have to reinitialize printer here
 Printer.Print ""
 Printer.ScaleMode = vbTwips
    
 ' Actual print to printer, starting from the user-selected Page No.
 fr.hdc = Printer.hdc
 fr.hdcTarget = Printer.hdc
    
 ' Update char range
 fr.mCharRange.firstChar = NextCharPosition

 Do
    If mPage > mToPage Then
        Exit Do
    End If
    ' Print the page by sending EM_FORMATRANGE message
    NextCharPosition = SendMessage(RTF.hwnd, EM_FORMATRANGE, True, fr)
    If NextCharPosition >= TextLength Then Exit Do 'If done then exit
    If mPage >= mToPage Then
        Exit Do
    End If
    fr.mCharRange.firstChar = NextCharPosition ' Starting position for next page
    Printer.NewPage  ' Move on to next page
    Printer.Print "" ' Re-initialize hDC
    Printer.Orientation = PrnOrient
    fr.hdc = Printer.hdc
    fr.hdcTarget = Printer.hdc
    mPage = mPage + 1
 Loop

 ' Commit the print job
 Printer.EndDoc

 '  
 R = SendMessage(RTF.hwnd, EM_FORMATRANGE, False, ByVal CLng(0))
End Sub

Public Sub PrintText(txt As String, size As Integer, B As Boolean, i As Boolean, U As Boolean, A As Integer)
 frmEdit.Text.Font.Charset = 204  '
 frmEdit.Text.SelFontSize = size
 frmEdit.Text.SelBold = B
 frmEdit.Text.SelItalic = i
 frmEdit.Text.SelUnderline = U
 frmEdit.Text.SelAlignment = A
 frmEdit.Text.SelText = txt
End Sub

